// ------------------------------------------------------------------------------
// AngularV1 - http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// AngularV1.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

// ---------------------------------------------------
// SETTINGS
// ---------------------------------------------------

int   SETTINGS_WIDTH                          = 800;
int   SETTINGS_HEIGHT                         = 420;
int   SETTINGS_MAXBULLETS                     = 20;
int   SETTINGS_MAXBOUNCE                      = 2;
int   SETTINGS_MAXRECTANGLES                  = 2;
int   SETTINGS_MAXTRAILS                      = 50;
float SETTINGS_CURSORSPEED                    = 7;
float SETTINGS_COMPUTERSPEED                  = 7;
float SETTINGS_COMPUTERACCELERATION           = 0.3;
float SETTINGS_CURSORSIZE                     = 17;
float SETTINGS_CURSORANGLE                    = 45;
float SETTINGS_BULLETSPEED                    = 20;
float SETTINGS_BOUNCECONSTANT                 = 1;
float SETTINGS_RATEOFFIRECOMPUTER             = 0.1;
float SETTINGS_RATEOFFIREHUMAN                = 0.1;
float SETTINGS_FRAMERATE                      = 30;
float SETTINGS_BULLETSPREAD                   = 2;
float SETTINGS_AIMINGCIRCLE                   = 35;
float SETTINGS_BULLETRECTANGLEPOWER           = 10;
float SETTINGS_BULLETCURSORPOWER              = 5;
float SETTINGS_BULLETLENGTH                   = 30;
float SETTINGS_XSHADOWDISTANCE                = 0;
float SETTINGS_YSHADOWDISTANCE                = 10;
float SETTINGS_SHIELDDECAY                    = 0.4;
float SETTINGS_SHIELDRADIUS                   = 25;
float SETTINGS_SHIELDSPEEDK                   = 0.4;
float SETTINGS_CHARGEDISCHARGERATIO           = 20;
float SETTINGS_SHAKEAMOUNT                    = 5;
float SETTINGS_COMPUTERROTATESPEED            = 10;
float SETTINGS_FUTURE                         = 15;
float SETTINGS_PADDING                        = 20;

// ---------------------------------------------------
// STATS
// ---------------------------------------------------

int STATS_CURRENTBULLET;
int STATS_CURRENTTRAIL;
int STATS_MODETIME;
int STATS_FRAME;
float STATS_HIDEX;
float STATS_HIDEY;
float STATS_BACKGROUNDOPACITY;
boolean STATS_ACTIVE = false;
String STATS_GAME;

// ---------------------------------------------------
// ARRAYS
// ---------------------------------------------------

Ray RAY                                       = new Ray();
Cursor[] CURSORS                              = new Cursor[2];
Bullet[] BULLETS                              = new Bullet[SETTINGS_MAXBULLETS];
Trail[] TRAILS                                = new Trail[SETTINGS_MAXTRAILS];
Rectangle[] RECTANGLES                        = new Rectangle[SETTINGS_MAXRECTANGLES];

// ---------------------------------------------------
// INTERFACE
// ---------------------------------------------------

PFont font;

void setup()
{
  size(800,420);
  framerate(SETTINGS_FRAMERATE);
  smooth();
  reset();

  font = loadFont("Verdana-Bold-14.vlw");
  textFont(font, 14);
}

void reset()
{
  CURSORS[0] = new Cursor(700,height/2,"human");
  CURSORS[1] = new Cursor(100,height/2,"computer");
  CURSORS[0].calculatePoints();
  CURSORS[1].calculatePoints();
  RECTANGLES[0] = new Rectangle(200, height/2, 30, 175, 0, 0.95);
  RECTANGLES[1] = new Rectangle(600, height/2, 30, 175, 0, 0.95);
  
  for ( int i=0; i<SETTINGS_MAXBULLETS; i++ )
  {
    BULLETS[i] = new Bullet(-10,-10,0,0,0,0);
  }
  
  for ( int i=0; i<SETTINGS_MAXTRAILS; i++ )
  {
    TRAILS[i] = new Trail(-10,-10,-10,-10);
  }
  
  STATS_CURRENTBULLET                       = 0;
  STATS_CURRENTTRAIL                        = 0;
  STATS_MODETIME                            = 0;
  STATS_FRAME                               = 0;
  STATS_HIDEX                               = 100;
  STATS_HIDEY                               = SETTINGS_HEIGHT/2;
  STATS_BACKGROUNDOPACITY                   = 255;
  STATS_GAME                                = "under";
}

void draw()
{
  checkKeyboard();
  updateBullets();
  updateRectangles();
  updateTrails();
  updateRay();
  if ( STATS_ACTIVE )
  {
    updateCursors();
    
    if ( STATS_GAME == "under" )
    {
      STATS_FRAME++;
    }
  }
  drawBackground();
  drawBullets();
  drawRectangles();
  drawTrails();
  drawCursors();
  //drawRay();
  drawGameOver();
}

// ---------------------------------------------------
// EFFECTS
// ---------------------------------------------------

void blur( float O )
{
  STATS_BACKGROUNDOPACITY = O;
}

void shake()
{
  translate(random(-SETTINGS_SHAKEAMOUNT,SETTINGS_SHAKEAMOUNT),random(-SETTINGS_SHAKEAMOUNT,SETTINGS_SHAKEAMOUNT));
}

// ---------------------------------------------------
// CALCULATION
// ---------------------------------------------------

void updateRay()
{
  RAY.update();
}

void updateRectangles()
{
  for ( int i=0; i<SETTINGS_MAXRECTANGLES; i++ )
  {
    RECTANGLES[i].update();
  }
}

void updateBullets()
{
  for ( int i=0; i<SETTINGS_MAXBULLETS; i++ )
  {
    BULLETS[i].update();
  }
}

void updateTrails()
{
  for ( int i=0; i<SETTINGS_MAXTRAILS; i++ )
  {
    TRAILS[i].update();
  }
}

void updateCursors()
{
  CURSORS[0].update();
  CURSORS[1].update();
}

// ---------------------------------------------------
// DRAWING
// ---------------------------------------------------

void drawGameOver()
{
  if ( STATS_GAME == "over" )
  {
    fill(255,255,255,230);
    stroke(255,255,255);
    rect(0,0,width,height);
    fill(0,0,0);
    textAlign(CENTER);
    text("GAME OVER",400,200);
    fill(50,50,50);
    if ( CURSORS[0].h == 0 )
    {
      text("You were annihilated in " + int(STATS_FRAME/SETTINGS_FRAMERATE) + " seconds",400,230);
    }
    else
    {
      text("You destroyed the guru in " + int(STATS_FRAME/SETTINGS_FRAMERATE) + " seconds",400,230);
    }
    text("Press space to try again",400,250);
  }
}

void drawBullets()
{
  for ( int i=0; i<SETTINGS_MAXBULLETS; i++ )
  {
    BULLETS[i].display();
  }
}

void drawRay()
{
  RAY.display();
}

void drawRectangles()
{
  for ( int i=0; i<SETTINGS_MAXRECTANGLES; i++ )
  {
    RECTANGLES[i].display();
  }
}

void drawTrails()
{
  for ( int i=0; i<SETTINGS_MAXTRAILS; i++ )
  {
    TRAILS[i].display();
  }
}

void drawCursors()
{
  CURSORS[0].display();
  CURSORS[1].display();
}

void drawBackground()
{
  if ( STATS_BACKGROUNDOPACITY < 240 )
  {
    STATS_BACKGROUNDOPACITY += ( 255 - STATS_BACKGROUNDOPACITY )/5;
    stroke(255,255,255);
    fill(255,255,255,STATS_BACKGROUNDOPACITY);
    rect(0,0,width,height);
  }
  else
  {
    background(255,255,255);
  }
}

// ---------------------------------------------------
// INPUT
// ---------------------------------------------------

void mousePressed()
{
  if ( !STATS_ACTIVE ) 
  {
    noCursor(); 
    STATS_ACTIVE = true; 
  }
  else
  {
    CURSORS[0].startShoot();
  }
}

void mouseReleased()
{
  CURSORS[0].stopShoot();
}

void checkKeyboard()
{
  if ( keyPressed )
  {
    if ( keyCode == SHIFT )
    {
      CURSORS[0].startShield();
    }
    if ( key == ' ' )
    {
      reset();
    }     
  }
}
