// ------------------------------------------------------------------------------
// AngularV1 - http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Bullet.java
// the bullets that the agents shoot
// ------------------------------------------------------------------------------

class Bullet
{
  int bounce;
  float x;
  float y;
  float vx;
  float vy;
  float r;
  float g;
  float b;
  boolean dead;
  
  Bullet( float X, float Y, float T, float R, float G, float B )
  {
    bounce = 0;
    x = X;
    y = Y;
    vx = SETTINGS_BULLETSPEED*cos(T);
    vy = SETTINGS_BULLETSPEED*sin(T);
    r = R;
    g = G;
    b = B;
  }
  
  void update()
  {
    if ( ! dead )
    {
      bounce();
      collide();
      move();
    }
  }
  
  void bounce()
  {
    if ( ( y + vy > height || y + vy < 0 ) && bounce < SETTINGS_MAXBOUNCE )
    {
      vy *= -SETTINGS_BOUNCECONSTANT;
      bounce++;
    }
    if ( ( x + vx > width || x + vx < 0 ) && bounce < SETTINGS_MAXBOUNCE )
    {
      vx *= -SETTINGS_BOUNCECONSTANT;
      bounce++;
    }
  }
  
  void collide()
  {
    for ( int i=0; i<SETTINGS_MAXRECTANGLES; i++ )
    {
      float x2 = RECTANGLES[i].cx;
      float y2 = RECTANGLES[i].cy;
      float a = RECTANGLES[i].a;
      float b = RECTANGLES[i].b;
        
      float deltax = x2 - x;
      float deltay = y2 - y;
      float deltad = sqrt(pow(deltax,2)+pow(deltay,2));
         
      if ( deltad < sqrt(pow(a/2,2)+pow(b/2,2)) )
      {
        if ( RECTANGLES[i].checkPos(x,y) )
        {
          float theta = atan2(vy,vx);
          float fx = SETTINGS_BULLETRECTANGLEPOWER*cos(theta);
          float fy = SETTINGS_BULLETRECTANGLEPOWER*sin(theta);
      
          RECTANGLES[i].exertForce(fx,fy,x-x2,y-y2);
          dead = true;
        }
      }
    } 
    for ( int i=0; i<2; i++ )
    {
      float theta = CURSORS[i].t;
      float deltax = CURSORS[i].x - SETTINGS_CURSORSIZE*cos(theta) - x;
      float deltay = CURSORS[i].y - SETTINGS_CURSORSIZE*sin(theta) - y;
      float deltad = sqrt(pow(deltax,2)+pow(deltay,2));
      
      float fdeltax = deltax - vx;
      float fdeltay = deltay - vy;
      float fdeltad = sqrt(pow(fdeltax,2)+pow(fdeltay,2));
      
      if ( fdeltad < CURSORS[i].sr && CURSORS[i].sr > 5 )
      {
        float ffdeltax = fdeltax - vx;
        float ffdeltay = fdeltay - vy;
        float ffdeltad = sqrt(pow(ffdeltax,2)+pow(ffdeltay,2));
      
        if ( ffdeltad < deltad )
        {
          float mtheta = atan2(-deltay,-deltax);
          float ctheta = atan2(-vy,-vx);
          float ftheta = mtheta + ( mtheta - ctheta );
        
          vx = SETTINGS_BULLETSPEED*cos(ftheta);
          vy = SETTINGS_BULLETSPEED*sin(ftheta);
          
          bounce++;
          STATS_MODETIME = 0;
        }
      }
      else if ( deltad < SETTINGS_CURSORSIZE )
      {
        CURSORS[i].reduceHealth(SETTINGS_BULLETCURSORPOWER);
        blur(50);
        shake();
        dead = true;
      }
    }      
  }
  
  void move()
  {
    x += vx;
    y += vy;
  }
  
  void display()
  {
    if ( !dead )
    {
      float theta = atan2(vy,vx);
     
      stroke(r,g,b);
      strokeWeight(2);
      line(x,y,x-SETTINGS_BULLETLENGTH*cos(theta)/2, y-SETTINGS_BULLETLENGTH*sin(theta)/2);
    }
  }
}
