// ------------------------------------------------------------------------------
// AngularV1 - http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Cursor.java
// The agents, good or bad.
// ------------------------------------------------------------------------------

class Cursor
{
  String type;
  float framecount;
  float x,y;
  float tx,ty;
  float vx,vy;
  float vd;
  float sd,st;
  float t;
  float h;
  float x1,y1,x2,y2;
  float px,py,px1,py1,px2,py2;
  float sr;
  float stheta;
  float sx1,sy1,sx2,sy2,sx3,sy3;
  float cx,cy;
  float k;
  boolean aim;
  boolean shoot;
  String reflect;
  String mode;

  Cursor( float X, float Y, String Type )
  {
    x = tx = px = X;
    y = ty = py = Y;
    t = 0;
    type = Type;
    h = 100;
    stheta = 0;
    k = 1;
    mode = "pursuit";
  }
  
  void update()
  {   
    if ( type == "human" )
    {
      setTargetHuman();
      moveHuman();
      if ( shoot && int(framecount*SETTINGS_RATEOFFIREHUMAN) == framecount*SETTINGS_RATEOFFIREHUMAN) { primaryShoot(); }
    }
    else
    {
       think();
       STATS_MODETIME++;
       if ( shoot && int(framecount*SETTINGS_RATEOFFIRECOMPUTER) == framecount*SETTINGS_RATEOFFIRECOMPUTER) { primaryShoot(); }
    }   

    calculatePoints();
    calculateShield();
    framecount++;
    makeTrail();
    px = x;
    py = y;
    px1 = x1;
    py1 = y1;
    px2 = x2;
    py2 = y2;
  }
  
  // ---------------------------------------------------
  // Human Cursor Functions
  // ---------------------------------------------------
  
  void setTargetHuman()
  {   
    float deltax = mouseX-x;
    float deltay = mouseY-y;
    float deltad = sqrt(pow(deltax,2)+pow(deltay,2));
    float theta = atan2(deltay,deltax);
    t = theta;
    
    if ( deltad > SETTINGS_AIMINGCIRCLE+5 )
    {
      tx = mouseX - SETTINGS_AIMINGCIRCLE*cos(theta);
      ty = mouseY - SETTINGS_AIMINGCIRCLE*sin(theta);
      aim = false;
    }
    else
    {
      tx = x;
      ty = y;
      aim = true;
    }
  } 
  
  void moveHuman()
  {
    float deltax = tx-x;
    float deltay = ty-y;
    
    vx = deltax*SETTINGS_CURSORSPEED/100;
    vy = deltay*SETTINGS_CURSORSPEED/100;
    
    float vd = sqrt(pow(vx,2)+pow(vy,2));
    
    if ( vd > k*SETTINGS_CURSORSPEED )
    {
      float theta = atan2(vy,vx);
      vx = k*SETTINGS_CURSORSPEED*cos(theta);
      vy = k*SETTINGS_CURSORSPEED*sin(theta);
    }
    
    x += vx;
    y += vy;
  }
   
  // ---------------------------------------------------
  // Computer Cursor Functions
  // ---------------------------------------------------
  
  void think()
  {
    if ( mode == "pursuit" )
    {   
      if ( RAY.crossed )
      {
        if ( shoot ) { stopShoot(); }
      }
      else
      {
        if ( !shoot ) { startShoot(); }
      }
      
      aimStraightComputer();
      moveComputer();
      if ( STATS_MODETIME > 3*SETTINGS_FRAMERATE || sqrt(pow(CURSORS[0].x - x,2)+pow(CURSORS[0].y - y,2)) < 50)
      {
        changeMode("hide");
      }
    }
    
    if ( mode == "turret" )
    {
      if ( RAY.crossed )
      {
        aimReflectComputer();
      }
      else
      {
        aimStraightComputer();
      }
      
      if ( !shoot ) { startShoot(); }
      haltComputer();
      if ( STATS_MODETIME > 5*SETTINGS_FRAMERATE )
      {
        changeMode("pursuit");
      }
    }
    
    if ( mode == "hide" )
    {
      if ( shoot ) { stopShoot(); }
      runComputer();  
    }
    
    if ( mode == "rest" )
    {
      if ( !RAY.crossed )
      {
        changeMode("turret");
      }
      aimStraightComputer();
      if ( shoot ) { stopShoot(); }
      haltComputer();
      if ( STATS_MODETIME > 1*SETTINGS_FRAMERATE )
      {
        changeMode("turret");
      }
    }
    
    if ( mode == "shield" )
    {
      if ( shoot ) { stopShoot(); }
      if ( STATS_MODETIME < SETTINGS_FRAMERATE )
      {
        startShield();
      }
      else
      {
        if ( sqrt(pow(CURSORS[0].x - x,2)+pow(CURSORS[0].y - y,2)) > 100 )
        {
          changeMode("rest");
        }
        else
        {
          changeMode("pursuit");
        }
      }
      aimStraightComputer();
      haltComputer();
    }
  }
  
  void changeMode( String Mode )
  {
    STATS_MODETIME = 0;
    mode = Mode;
  }
  
  void runComputer()
  {
    if ( CURSORS[0].x > 3*width/4 && !RAY.crossed )
    {
      STATS_HIDEX = 100;
      STATS_HIDEY = height/2+random(-100,100);
    }
    else if ( CURSORS[0].x < width/4 && !RAY.crossed)
    {
      STATS_HIDEX = 700;
      STATS_HIDEY = height/2+random(-100,100);
    }
    
    if ( abs(STATS_HIDEX-x) > 20 || abs(STATS_HIDEY-y) > 20 )
    {
      setSteer(STATS_HIDEX, STATS_HIDEY);  
      moveComputer();
    }
    else
    {
      changeMode("rest");
    }
  }
  
  void haltComputer()
  {
    float sx = sd*cos(st+t);
    float sy = sd*sin(st+t);
    
    vx = vd*cos(t);
    vy = vd*sin(t);
    
    vx += sx;
    vy += sy;
    
    vd = sqrt(pow(vx,2)+pow(vy,2));
    t = atan2(vy,vx);
    
    vd = 1;
  }
  
  void moveComputer()
  {
    float sx = sd*cos(st+t);
    float sy = sd*sin(st+t);
    
    vx = vd*cos(t);
    vy = vd*sin(t);
    
    vx += sx;
    vy += sy;
    
    vd = sqrt(pow(vx,2)+pow(vy,2));
    t = atan2(vy,vx);
    
    if ( vd < k*SETTINGS_COMPUTERSPEED )
    {
      vd += SETTINGS_COMPUTERACCELERATION;
    }
    
    if ( vd > k*SETTINGS_COMPUTERSPEED )
    {
      vd = k*SETTINGS_COMPUTERSPEED;
      vx = vd*cos(t);
      vy = vd*sin(t);
    }
    
    x += vx;
    y += vy;
  }
  
  void aimStraightComputer()
  {
    float deltax = CURSORS[0].x - x;
    float deltay = CURSORS[0].y - y;
    float deltad = sqrt(pow(deltax,2)+pow(deltay,2));
    float timeToCollision = deltad/SETTINGS_BULLETSPEED;
    float deltaxf = CURSORS[0].x + CURSORS[0].vx*timeToCollision - x;
    float deltayf = CURSORS[0].y + CURSORS[0].vy*timeToCollision - y;
    float deltadf = sqrt(pow(deltaxf,2)+pow(deltayf,2));
    float timeToCollisionF = deltadf/SETTINGS_BULLETSPEED;
    float xTarget = CURSORS[0].cx + CURSORS[0].vx*timeToCollisionF;
    float yTarget = CURSORS[0].cy + CURSORS[0].vy*timeToCollisionF;
    
    setSteer(xTarget, yTarget);
  }
  
  void aimReflectComputer()
  {
    float deltax = CURSORS[0].cx - x;
    float deltay = CURSORS[0].cy - y;
    float dx;
    float xTarget;
    float yTarget;
    
    if ( CURSORS[0].y < height/4 )
    {
      reflect = "top";
    }
    else if ( CURSORS[0].y > 3*height/4 )
    {
      reflect = "bottom";
    }
      
    if ( reflect == "top" )
    {
      dx = deltax/(1+y/CURSORS[0].cy);
      xTarget = CURSORS[0].cx - dx;
      yTarget = 0;
    }
    else
    {
      dx = deltax/(1+(height-y)/(height-CURSORS[0].cy));
      xTarget = CURSORS[0].cx - dx;
      yTarget = height;
    }
    
    setSteer(xTarget, yTarget);
  }
  
  void setSteer( float X, float Y )
  {
    if ( isLeft(x,y,x+vx,y+vy,X,Y) > 0 )
    {
      st = PI/2;
    }
    else
    {
      st = -PI/2;
    }
    
    float d = sqrt(pow(X-x,2)+pow(Y-y,2));
    float xf = x + d*cos(t);
    float yf = y + d*sin(t);
    
    sd = sqrt(pow(X-xf,2)+pow(Y-yf,2))*SETTINGS_COMPUTERROTATESPEED/(10*d);
  }
  
  float isLeft( float x1, float y1, float x2, float y2, float x3, float y3 )
  {
    return (x2 - x1)*(y3 - y1) - (x3 - x1)*(y2 - y1);
  }
  
  // ---------------------------------------------------
  // Common Functions
  // ---------------------------------------------------
  
  void die()
  {
    vx = 0;
    vy = 0.5;
  }
  
  void calculatePoints()
  {
    x1 = SETTINGS_CURSORSIZE*cos(t-radians(SETTINGS_CURSORANGLE/2));
    y1 = SETTINGS_CURSORSIZE*sin(t-radians(SETTINGS_CURSORANGLE/2));
    x2 = SETTINGS_CURSORSIZE*cos(t+radians(SETTINGS_CURSORANGLE/2));
    y2 = SETTINGS_CURSORSIZE*sin(t+radians(SETTINGS_CURSORANGLE/2));
    cx = x-SETTINGS_CURSORSIZE*cos(t)/2;
    cy = y-SETTINGS_CURSORSIZE*sin(t)/2;
  }
  
  void calculateShield()
  {
    if ( sr - SETTINGS_SHIELDDECAY >= 0 )
    {
      sr -= SETTINGS_SHIELDDECAY;
    }
    if ( k + SETTINGS_SHIELDDECAY/SETTINGS_SHIELDRADIUS < 1 )
    {
      k += SETTINGS_SHIELDDECAY/SETTINGS_SHIELDRADIUS;
    }
    
    stheta += radians(15);
    
    sx1 = x - SETTINGS_CURSORSIZE*cos(t)/2 + sr*cos(stheta);
    sy1 = y - SETTINGS_CURSORSIZE*sin(t)/2 + sr*sin(stheta);
    sx2 = x - SETTINGS_CURSORSIZE*cos(t)/2 + sr*cos(stheta+2*PI/3);
    sy2 = y - SETTINGS_CURSORSIZE*sin(t)/2 + sr*sin(stheta+2*PI/3);
    sx3 = x - SETTINGS_CURSORSIZE*cos(t)/2 + sr*cos(stheta+4*PI/3);
    sy3 = y - SETTINGS_CURSORSIZE*sin(t)/2 + sr*sin(stheta+4*PI/3);
  }
  
  void primaryShoot()
  {
    float r,g,b;
    
    if ( type == "human" ) 
    { 
      r = 253;
      g = 33;
      b = 33;
    }
    else
    {
      r = 253;
      g = 157;
      b = 33;
    }
    
    BULLETS[STATS_CURRENTBULLET] = new Bullet(x+10*cos(t),y+10*sin(t),t+radians(random(-SETTINGS_BULLETSPREAD/2,SETTINGS_BULLETSPREAD/2)),r,g,b);
    
    STATS_CURRENTBULLET++;
    
    if ( STATS_CURRENTBULLET == SETTINGS_MAXBULLETS )
    {
      STATS_CURRENTBULLET = 0;
    }
  }
  
  void makeTrail()
  {
    TRAILS[STATS_CURRENTTRAIL] = new Trail(x-x1,y-y1,px-px1,py-py1);
    STATS_CURRENTTRAIL++;
    TRAILS[STATS_CURRENTTRAIL] = new Trail(x-x2,y-y2,px-px2,py-py2);
    STATS_CURRENTTRAIL++;
    
    if ( STATS_CURRENTTRAIL == SETTINGS_MAXTRAILS )
    {
      STATS_CURRENTTRAIL = 0;
    }
  }
  
  void startShoot()
  {
    if ( sr < 5 )
    {
      shoot = true;
    }

    framecount = 0;
  }
  
  void stopShoot()
  {
    shoot = false;
  }
  
  void startShield()
  {
    if ( sr  + SETTINGS_SHIELDDECAY*SETTINGS_CHARGEDISCHARGERATIO < SETTINGS_SHIELDRADIUS )
    {
      sr += SETTINGS_SHIELDDECAY*SETTINGS_CHARGEDISCHARGERATIO;
    }
    else
    {
      sr = SETTINGS_SHIELDRADIUS;
    }
    
    shoot = false;
    k = 1 - (1-SETTINGS_SHIELDSPEEDK)*sr/SETTINGS_SHIELDRADIUS;
  }
  
  void reduceHealth( float H )
  {
    if ( h - H > 0 )
    {
      h -= H;
      if ( mode != "shield" ) { changeMode("shield"); }
    }
    else
    {
      h = 0;
      STATS_GAME = "over";
    }
  }
  
  // ---------------------------------------------------
  // Draw Functions
  // ---------------------------------------------------

  void display()
  {
    drawShadow();
    drawShield(); 
    drawTarget();
    drawAfterBurner();
    drawCursor();
    drawHealth();
  }
  
  void drawTarget()
  {
    if ( type == "human" )
    {
      if ( aim && sr < 5 )
      {
        stroke(50,200,50,100);
        strokeWeight(1);
        line(x,y,mouseX,mouseY);
        
        stroke(50,200,50);
        strokeWeight(3);
        point(mouseX,mouseY);
      }
      
      stroke(200,50,50);
      strokeWeight(4);
      point(tx,ty);
    }
  }
  
  void drawCursor()
  {  
    stroke(0,0,0);
    strokeWeight(2);
    fill(255,255,255);
    
    beginShape(POLYGON);
    vertex(x,y);
    vertex(x-x1,y-y1);
    vertex(x-x2,y-y2);
    vertex(x,y);
    endShape();
  }
  
  void drawShadow()
  {  
    stroke(0,0,0,10);
    strokeWeight(2);
    fill(0,0,0,40);
    
    beginShape(POLYGON);
    vertex(x+SETTINGS_XSHADOWDISTANCE,y+SETTINGS_YSHADOWDISTANCE);
    vertex(x-x1+SETTINGS_XSHADOWDISTANCE,y-y1+SETTINGS_YSHADOWDISTANCE);
    vertex(x-x2+SETTINGS_XSHADOWDISTANCE,y-y2+SETTINGS_YSHADOWDISTANCE);
    vertex(x+SETTINGS_XSHADOWDISTANCE,y+SETTINGS_YSHADOWDISTANCE);
    endShape();
  }
  
  void drawAfterBurner()
  {
    float offset = radians(SETTINGS_CURSORANGLE/3);
    float x1 = x-0.9*SETTINGS_CURSORSIZE*cos(t+offset);
    float y1 = y-0.9*SETTINGS_CURSORSIZE*sin(t+offset);
    float x2 = x-0.9*SETTINGS_CURSORSIZE*cos(t-offset);
    float y2 = y-0.9*SETTINGS_CURSORSIZE*sin(t-offset);
    float r = sqrt(pow(vx,2)+pow(vy,2))/5;
    float x3 = x-r*SETTINGS_CURSORSIZE*cos(t);
    float y3 = y-r*SETTINGS_CURSORSIZE*sin(t);
    
    strokeWeight(2);
    stroke(234,78,17);
    fill(234,175,17);
    beginShape(POLYGON);
    vertex(x1,y1);
    vertex(x2,y2);
    vertex(x3,y3);
    vertex(x1,y1);
    endShape();
  }
  
  void drawHealth()
  {
    float xh1 = (100-h)/100*x1;
    float yh1 = (100-h)/100*y1;
    float xh2 = (100-h)/100*x2;
    float yh2 = (100-h)/100*y2;
    
    stroke(0,0,0);
    strokeWeight(1);
    fill(255,50,50);
    
    beginShape(POLYGON);
    vertex(x,y);
    vertex(x-xh1,y-yh1);
    vertex(x-xh2,y-yh2);
    vertex(x,y);
    endShape();
  }
  
  void drawShield()
  {
    if ( sr > 5 ) 
    { 
      stroke(202,36,169,50);
      fill(255,255,255,0);
      strokeWeight(2);
      ellipse(cx,cy,2*sr,2*sr);
      stroke(41,88,136);
      strokeWeight(4);
      point(sx1,sy1);
      point(sx2,sy2);
      point(sx3,sy3);
    }
  }
}
