// ------------------------------------------------------------------------------
// AngularV1 - http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Square.java
// The obstacles that have physical properties... yummm...
// ------------------------------------------------------------------------------

class Rectangle
{
  float i;
  float w;
  float m;
  float a;
  float b;
  float vx;
  float vy;
  float cx;
  float cy;
  float x1, y1, x2, y2, x3, y3, x4, y4;
  float rdamp;
  float vdamp;
  float omega;
  
  Rectangle(float CX, float CY, float A, float B, float VDAMP, float RDAMP )
  {
    cx = CX;
    cy = CY;
    a = A;
    b = B;
    omega = 0;
    m = 10;
    w = 0;
    i = m*(pow(a,2) + pow(b,2))/12;
    vdamp = VDAMP;
    rdamp = RDAMP;
  }
  
  void update()
  {
    moveBox();
    rotateBox();
  }
  
  void exertForce(float FX, float FY, float X, float Y)
  {
    float l = sqrt(pow(FX,2)+pow(FY,2))*sqrt(pow(X,2)+pow(Y,2))*sin(atan2(FY,FX)-atan2(Y,X));
      
    vx += FX/m;
    vy += FY/m;
    w  += -l/i;
  }
  
  void rotateBox()
  {
    omega += w;
    w *= rdamp;
    
    float cw = cos(omega)*a/2;
    float sw = sin(omega)*a/2;
    float ch = cos(omega)*b/2;
    float sh = sin(omega)*b/2;
    
    x1 = + cw + sh;
    y1 = - sw + ch;
    x2 = + cw - sh;
    y2 = - sw - ch;
    x3 = - cw - sh;
    y3 = + sw - ch;
    x4 = - cw + sh;
    y4 = + sw + ch;    
  }
  
  void moveBox()
  {
    vx *= vdamp;
    vy *= vdamp;
    
    cx += vx;
    cy += vy;
  }
  
  boolean checkPos( float X, float Y )
  {
    float mx = cos(-omega)*(X-cx)+sin(-omega)*(Y-cy);
    float my = cos(-omega)*(Y-cy)-sin(-omega)*(X-cx);
     
    if ( abs(mx) < a/2 && abs(my) < b/2 )
    {
      return true;     
    }
    else
    {
      return false;
    }
  }
  
  boolean checkPosApprox( float X, float Y )
  {
    float mx = cos(-omega)*(X-cx)+sin(-omega)*(Y-cy);
    float my = cos(-omega)*(Y-cy)-sin(-omega)*(X-cx);
     
    if ( abs(mx) - 5 < a/2 && abs(my) - 5 < b/2 )
    {
      return true;     
    }
    else
    {
      return false;
    }
  }
  
  void display()
  {
    stroke(58,117,159);
    fill(231,238,243);
    strokeWeight(1);
    beginShape(POLYGON); 
    vertex(cx+x1,cy+y1);
    vertex(cx+x2,cy+y2);
    vertex(cx+x3,cy+y3);
    vertex(cx+x4,cy+y4);
    endShape(); 
    strokeWeight(3);
    point(cx,cy);
  }
}
